"use client";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useEffect, useMemo, useState } from "react";
import "./clock.css";
import { defaultFill } from "./utils";
var OPTIONS = {
    size: 400,
    face: {
        background: "white",
        show: true,
        padding: 0,
        ticks: {
            show: true,
            regular: {
                show: true,
                height: 20,
                width: 4,
                background: "rgb(200 200 200)",
                radius: 2,
            },
            secondary: {
                show: true,
                height: 28,
                width: 4,
                background: "black",
                radius: 2,
            },
            primary: {
                show: true,
                height: 36,
                width: 8,
                background: "rgb(0 122 255)",
                radius: 3,
            },
        },
        counts: {
            show: true,
            secondary: {
                show: true,
                size: 20,
                color: "rgb(100 100 100)",
                background: "rgb(0 122 255 / 0)",
                gap: 5,
            },
            primary: {
                show: true,
                size: 25,
                color: "black",
                background: "rgb(0 122 255 / 0.08)",
                gap: 5,
            },
        },
    },
    interface: {
        show: true,
        dynamic: true,
        transition: 300,
        pivot: {
            show: true,
            size: 20,
            background: "rgb(0 0 0)",
        },
        hourHand: {
            show: true,
            front: {
                show: true,
                background: "black",
                radius: 4,
                width: 12,
                height: 80,
                alignment: "CENTER",
            },
            frontBase: {
                show: true,
                background: "grey",
                radius: 2,
                width: 4,
                height: 120,
            },
            back: {
                show: true,
                background: "grey",
                radius: 2,
                width: 4,
                height: 24,
            },
        },
        minuteHand: {
            show: true,
            front: {
                show: true,
                background: "black",
                radius: 4,
                width: 8,
                height: 120,
                alignment: "TICK",
            },
            frontBase: {
                show: true,
                background: "grey",
                radius: 2,
                width: 4,
                height: 150,
            },
            back: {
                show: true,
                background: "grey",
                radius: 2,
                width: 4,
                height: 30,
            },
        },
        secondHand: {
            show: true,
            front: {
                show: true,
                background: "red",
                radius: 4,
                width: 6,
                height: 120,
                alignment: "CENTER",
            },
            frontBase: {
                show: true,
                background: "orange",
                radius: 2,
                width: 4,
                height: 150,
            },
            back: {
                show: true,
                background: "grey",
                radius: 2,
                width: 4,
                height: 30,
            },
        },
    },
};
var Tick = function (_a) {
    var radius = _a.radius, background = _a.background;
    return (React.createElement("div", { className: "tick", style: {
            "--radius": "".concat(radius, "px"),
            "--background": "".concat(background),
        } }));
};
var Count = function (_a) {
    var number = _a.number, counts = _a.counts, level = _a.level;
    var countByLevel = counts[level];
    if (!countByLevel.show)
        return null;
    return (React.createElement("span", { className: "tick-count", style: {
            "--color": "".concat(countByLevel.color),
            "--background": "".concat(countByLevel.background),
            "--size": "".concat(countByLevel.size, "px"),
            "--gap": "".concat(countByLevel.gap, "px"),
        } }, number));
};
var PerimeterPlacer = function (_a) {
    var ticks = _a.ticks, apparentClockRadius = _a.apparentClockRadius, counts = _a.counts;
    var totalTicks = 60;
    var degPerTick = 360 / totalTicks;
    var primaries = new Set([0, 3, 6, 9]);
    var levelByIndex = ["regular", "secondary", "primary"];
    var tickByLevelIndex = [];
    for (var i = 0; i < 3; i++) {
        var tick = ticks[levelByIndex[i]];
        tickByLevelIndex.push(tick);
    }
    // CTH : center to tick hypotenuse
    var cthByLevelIndex = [];
    for (var i = 0; i < 3; i++) {
        var tick = tickByLevelIndex[i];
        cthByLevelIndex.push(apparentClockRadius - tick.height / 2);
    }
    var mapper = new Array(totalTicks).fill(0);
    var showTicks = ticks.show;
    var showRegularTicks = showTicks && tickByLevelIndex[0].show;
    return (React.createElement(React.Fragment, null, mapper.map(function (_, i) {
        var deg = degPerTick * i;
        var levelIndex = primaries.has(i / 5) ? 2 : i % 5 === 0 ? 1 : 0;
        var cth = cthByLevelIndex[levelIndex];
        if (levelIndex === 0 && !showRegularTicks)
            return null;
        return (React.createElement("div", { className: "perimeter-placer", "data-tick-index": i, key: i, style: {
                "--center-to-tick-hypotenuse": "".concat(cth, "px"),
                "--degree": "".concat(deg, "deg"),
                "--tick-height": "".concat(showTicks && tickByLevelIndex[levelIndex].show
                    ? tickByLevelIndex[levelIndex].height
                    : 0, "px"),
                "--tick-width": "".concat(showTicks && tickByLevelIndex[levelIndex].show
                    ? tickByLevelIndex[levelIndex].width
                    : 0, "px"),
            } },
            React.createElement(Tick, { radius: tickByLevelIndex[levelIndex].radius, background: tickByLevelIndex[levelIndex].background }),
            counts.show && levelIndex !== 0 && (React.createElement(Count, { number: (i === 0 ? 60 : i) / 5, counts: counts, level: levelIndex === 1 ? "secondary" : "primary" }))));
    })));
};
var ClockHand = function (_a) {
    var _b;
    var clockHand = _a.clockHand, degrees = _a.degrees, pivotSize = _a.pivotSize, transition = _a.transition, props = __rest(_a, ["clockHand", "degrees", "pivotSize", "transition"]);
    var handSubParts = useMemo(function () { return ["frontBase", "back", "front"]; }, []);
    var handSubPartClassNames = useMemo(function () { return ["front-base", "back", "front"]; }, []);
    var _c = clockHand.frontBase, frontBaseHeight = _c.height, frontBaseWidth = _c.width;
    var _d = clockHand.front, frontHeight = _d.height, alignment = _d.alignment;
    var _e = clockHand.back, backHeight = _e.height, backWidth = _e.width;
    var frontAlignmentTranslation = useMemo(function () {
        return alignment === "PIVOT"
            ? frontHeight - frontBaseHeight
            : alignment === "CENTER"
                ? (frontHeight + pivotSize / 2 - frontBaseHeight) / 2
                : 0;
    }, [alignment, frontHeight, frontBaseHeight, pivotSize]);
    var resolvedHandWidth = useMemo(function () { return Math.max(frontBaseWidth, backWidth); }, [frontBaseWidth, backWidth]);
    return (React.createElement("div", __assign({}, props, { style: __assign({ "--hand-height": "".concat(backHeight + frontBaseHeight, "px"), "--hand-width": "".concat(resolvedHandWidth, "px"), "--hand-back-height": "".concat(backHeight, "px"), "--hand-rotate": "".concat(degrees, "deg"), "--transition": "".concat(transition, "ms"), "--hand-origin": "50% calc(100% - ".concat(backHeight, "px)") }, ((_b = props.style) !== null && _b !== void 0 ? _b : {})) }), handSubParts.map(function (handSubPart, i) {
        return (React.createElement("div", { className: handSubPartClassNames[i], key: i, "aria-hidden": !clockHand[handSubPart].show, style: __assign({ opacity: clockHand[handSubPart].show ? "" : 0, pointerEvents: clockHand[handSubPart].show ? "" : "none", "--hand-background": clockHand[handSubPart].background, "--hand-radius": "".concat(clockHand[handSubPart].radius, "px"), "--hand-width": "".concat(clockHand[handSubPart].width, "px"), "--hand-height": "".concat(clockHand[handSubPart].height, "px") }, (handSubPart === "front"
                ? {
                    "--hand-translate": "".concat(-frontAlignmentTranslation, "px"),
                }
                : {})) }));
    })));
};
var ClockInterface = function (_a) {
    var clockInterface = _a.clockInterface, time = _a.time;
    var _b = useState({
        hour: 0,
        minute: 0,
        second: 0,
    }), degRotations = _b[0], setDegRotations = _b[1];
    var dynamic = clockInterface.dynamic, pivot = clockInterface.pivot;
    var pivotSize = pivot.show ? pivot.size : 0;
    useEffect(function () {
        if (!dynamic)
            return;
        // Initialize the rotation degrees based on the initial time prop
        var initialHourDeg = ((time.getHours() % 12) / 12) * 360 + (time.getMinutes() / 60) * 30;
        var initialMinuteDeg = (time.getMinutes() / 60) * 360 + (time.getSeconds() / 60) * 6;
        var initialSecondDeg = (time.getSeconds() / 60) * 360;
        setDegRotations({
            hour: initialHourDeg,
            minute: initialMinuteDeg,
            second: initialSecondDeg,
        });
        // Update the rotation every second
        var interval = setInterval(function () {
            setDegRotations(function (prev) { return ({
                hour: prev.hour + 1 / 120, // 0.5 degrees per minute (30° per hour)
                minute: prev.minute + 0.1, // 6 degrees per minute
                second: prev.second + 6, // 360° per minute
            }); });
        }, 1000);
        return function () { return clearInterval(interval); };
    }, [time, dynamic]);
    return (React.createElement("div", { className: "interface" },
        Object.keys(degRotations).map(function (handName) {
            if (!clockInterface["".concat(handName, "Hand")].show)
                return null;
            return (React.createElement(ClockHand, { clockHand: clockInterface["".concat(handName, "Hand")], degrees: degRotations[handName], pivotSize: pivotSize, transition: clockInterface.transition, className: "".concat(handName, "-hand clock-hand"), key: handName }));
        }),
        pivot.show && (React.createElement("div", { className: "pivot", style: {
                "--pivot-height": "".concat(pivot.size, "px"),
                "--pivot-background": "".concat(pivot.background),
            } }))));
};
var Clock = function (_a) {
    var _b;
    var options = _a.options, time = _a.time, children = _a.children, props = __rest(_a, ["options", "time", "children"]);
    var normalizedOptions = defaultFill(OPTIONS, options !== null && options !== void 0 ? options : {});
    var size = normalizedOptions.size, _c = normalizedOptions.face, ticksOptions = _c.ticks, countsOptions = _c.counts, padding = _c.padding, background = _c.background, interfaceOptions = normalizedOptions.interface;
    var apparentClockRadius = size / 2 - padding;
    var _d = useState(time ? time : new Date()), date = _d[0], setDate = _d[1];
    useEffect(function () {
        setDate(time !== null && time !== void 0 ? time : new Date());
        if (time)
            return;
        var handleVisibilityChange = function () {
            console.log("Visibility change");
            if (document.hidden)
                return;
            console.log("Clock updated");
            setDate(new Date());
        };
        document.addEventListener("visibilitychange", handleVisibilityChange);
        return function () {
            document.removeEventListener("visibilitychange", handleVisibilityChange);
        };
    }, [time]);
    return (React.createElement("div", __assign({}, props, { className: "--custom-analog-clock-react ".concat(props.className), style: __assign({ "--clock-size": "".concat(size, "px"), "--clock-background": background }, ((_b = props.style) !== null && _b !== void 0 ? _b : {})) }),
        children ? children : null,
        normalizedOptions.face.show && (React.createElement(PerimeterPlacer, { ticks: ticksOptions, apparentClockRadius: apparentClockRadius, counts: countsOptions })),
        interfaceOptions.show && (React.createElement(ClockInterface, { clockInterface: interfaceOptions, time: date }))));
};
export default Clock;
